package derez.libs;

import java.text.ParseException;

import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.keywords.DateTime;


/**
 * The library provides data about the Sun and the Moon
 * like the times of Sunrise, Sunset, Moon rise and Moon set,
 * and more, according to place and date.
 * for the phase of the moon there are two algorithms, one is based on the 
 * day of the moon month in the Hebrew Calendar, the other on the Julian calendar.
 * The Sunrise and Sunset times vary according to the value of Zenith angle. 
 * The following conventions are in use:
 * 		Official       90.8333 (90 50')
 * 		Civil -        96
 * 		Nautical -     102
 * 		Astronomical - 108
 * The values can be used by their names to be selected 
 * using the method Zenith.
 * The library uses the Official value as default, 
 *  
 * The rise an set results were checked against this site
 * <link>Sun & Moon Tables | http://aa.usno.navy.mil/data/docs/RS_OneYear.php</link>
 * and found accurate within one minute difference.
 */

@ShortName("Astro")
@Author("Derez")
@Version((float)1.4)

public class Astro {
	
	public double OFFICIAL  = 90.83333333 ;
	public double CIVIL =  96 ;
	public double NAUTICAL =  102;
	public double ASTRONOMICAL =  108 ;
	private double zenith = OFFICIAL  ; 
	
	private String[] RiseSet = new String[2];
	double [][] Ma = new double[4][4];
	double F,T,B5,L5,Z0,M8=0,W8=0,A,A2=0,A0,D1,D2=0,D0,V0=0 ;
	double 	V1,V2=0,J,J3= 0,D5,R5=0,S,C,Z,T0 ;
	double L,M,V,W,N,U,A5,Z1,P,F0,F1,F2,B,L0,L2,H0,H1,H2 ;
	double E, T3,H3=0,M3=0,D,Mo,Y,G,C0;
	double P2 = 2 * Math.PI ;
	double DtoR = Math.PI / 180;
	double K1 = 15 * Math.PI * 1.0027379 / 180;
	
	
	/**
	 *  Gets and sets the value of zenith.
	 */
	public double getZenith()
	{
		return zenith ;
	}
	public void setZenith(double value) 
	{
		zenith = value ;	
	}
	
	/**
	 * Returns the phase of the moon as per the Hebrew calendar moon month.
	 */
	public int HebMoonPhase( int day, int month, int year) throws ParseException 
	{   // gets the int Hebrew day of the month only
		// test the input
		if ((day < 1 )|(day > 31)| (month<1)| (month>12)| (year <100))
			{return -1;}
		else if ((year == 1582) & (month == 10) & (day > 4) & (day < 15))
			{return -1;}
		else
		{
		DateTime.setDateFormat("dd/MM/yyyy") ;
		long dGreg = DateTime.DateParse("01/01/2000") + DateTime.TicksPerHour  ;
		dGreg = DateTime.Add(dGreg, year-2000, month-1, day-1) ; 
		return GToH(dGreg,day,month,year);
		}
	}
			
	private int GToH(long dGreg, int gday,int gmonth, int gyear) throws ParseException
	{	
		// Converts Gregorian date to Hebrew
		int nMonthH ;
		int nDateH ;
		boolean bHaser ;
		boolean bShalem ;
		int nMonthLen = 0 ;
	    boolean bWhile  ;
		int	nOneMolad = 29 + (12 / 24) + (793 / (1080 * 24)) ;
		int	nAvrgYear = nOneMolad * (235 / 19) ;
		int nDays = (int)((dGreg - DateTime.DateParse("01/01/1900"))/DateTime.TicksPerDay) ;
	
		nDays +=  2067025  ;  // 2067025 days after creation
		int nYearH = (nDays / nAvrgYear) + 1 ;
		long dTishrei1 = Tishrei1(nYearH);
					
		if (dTishrei1 == dGreg)
		  {
		     nMonthH = 1;
		     nDateH = 1;
		  }
		else
		  {
			if (dTishrei1 < dGreg)
			{
				while (Tishrei1(nYearH + 1) <= dGreg)
				 {nYearH++  ;}
		 	}
			else
			{
			     nYearH-- ;
			     while (Tishrei1(nYearH) > dGreg)
			        {nYearH--;}
			}
		  }
		
		nDays = (int)((dGreg - Tishrei1(nYearH))/ DateTime.TicksPerDay) ;
	    int nLengthOfYear = LengthOfYear(nYearH);
	    if ((nLengthOfYear == 353) | (nLengthOfYear == 383))
	    {  bHaser = true; }
	    else
	    {  bHaser = false;}
	    if ((nLengthOfYear == 355) | (nLengthOfYear == 385))
	    {  bShalem = true ;}
	    else
	    {  bShalem = false ;}
		boolean bLeap = IsLeapH(nYearH);
	    nMonthH = 1 ;
	    
	    do 
	    {
		    switch (nMonthH)
	  		{
		     case 1:
		     case 5:
		     case 8:
		     case 10:
		     case 12:
		    	  nMonthLen =  30;  break;   // 30 day months
		     case 4:
		     case 7:
		     case 9:
		     case 11:
		     case 13:
		    	  nMonthLen =  29;  break;
		     case 6:
		    	  nMonthLen = 30 ; break; //Adar A (6) will be skipped on non-leap years
		     case 2:                         // Cheshvan, see note above
		       	if (bShalem) { nMonthLen =  30;} else {nMonthLen =  29;}
		       	  break ;
		     case 3:                        // Kislev, see note above
		        if (bHaser) { nMonthLen =  29;} else {nMonthLen = 30;}
		     	break ;
		    }    
		    if (nDays >= nMonthLen)
		    {
		      bWhile = true;
		      if (bLeap | (nMonthH != 5))
		        {nMonthH++ ;}
		      else
		          {nMonthH +=  2;}
		      nDays -= nMonthLen;
		    }
		    else
		    { bWhile = false;}
	    }
	    while ( bWhile) ;
	    	{nDateH = nDays + 1;}
	    return nDateH ;
	   
	} 
	
	private long Tishrei1(int nYearH) throws ParseException 
	{
		int nMonthsSinceFirstMolad = MonSinceFirstMolad(nYearH);
		int nChalakim = 793 * nMonthsSinceFirstMolad ;
		nChalakim +=  204 ;
		int nHours = nChalakim / 1080 ;
		nChalakim = nChalakim % 1080 ;
		
		nHours +=  nMonthsSinceFirstMolad * 12 ;
		nHours +=  5;
		    
		int nDays = nHours / 24 ;
		nHours = nHours % 24 ;
		
		nDays += 29 * nMonthsSinceFirstMolad ;
		nDays +=  2 ;
		
		int nDayOfWeek = nDays % 7 ;
		
		if ((!IsLeapH(nYearH)) & ( nDayOfWeek == 3) & ((nHours * 1080 + nChalakim) >= (9 * 1080 + 204)))
		  {
		   nDayOfWeek = 5 ;
		   nDays +=  2 ;
		  }
		else
		  {
			if ((IsLeapH(nYearH - 1)) & (nDayOfWeek == 2) & ((nHours * 1080 + nChalakim) >= (15 * 1080 + 589))) 
			  {
				nDayOfWeek = 3 ;
		        nDays++ ;
			  }
		   else
		      {
		   		if (nHours >= 18) 
		   	 	 {
		          nDayOfWeek++;
		          nDayOfWeek %=  7 ;
		          nDays++  ;
		     	 }
		   		if ((nDayOfWeek == 1) | (nDayOfWeek == 4) | (nDayOfWeek == 6)) 
		     	 {
		          nDayOfWeek++  ;
		          nDayOfWeek %=  7;
		          nDays++  ;
		   	 	 }
		      }
		  }
		nDays -=  2067025 ;
		long dTishrei1 = DateTime.DateParse("01/01/1900") ;
		return dTishrei1 + nDays * DateTime.TicksPerDay ;
	}
		
	private boolean IsLeapH(int hyear )
	{
		hyear = hyear % 19 ;
		if((hyear==0)|(hyear==3)|(hyear==6)|(hyear==8)|(hyear==11)|(hyear==14)|(hyear==17))
			{ return true ;}
		else
		    { return false ;}
	}
	
	private int LengthOfYear(int nYearH ) throws ParseException 
	{
		long dThisTishrei1 = Tishrei1(nYearH);
		long dNextTishrei1 = Tishrei1(nYearH + 1);
   		return (int)((dNextTishrei1 - dThisTishrei1)/ DateTime.TicksPerDay) ;
	}

	private int MonSinceFirstMolad(int nYearH)  
	{
			nYearH  -=  1 ;
			int nMonSinceFirstMolad = (nYearH / 19) * 235 ;
			nYearH %=  19 ;
			nMonSinceFirstMolad +=  12 * nYearH ;
			if (nYearH >= 17) 
			{nMonSinceFirstMolad +=  6 ;}
			else
			{
			  if (nYearH >= 14) 
				{ nMonSinceFirstMolad +=  5;}
			  else
			    {
			  	if (nYearH >= 11) 
			  	{nMonSinceFirstMolad +=  4;}
			    else
			      {
			    	if (nYearH >= 8) 
			    	{nMonSinceFirstMolad +=  3;}
			      else
			        {
			      	if (nYearH >= 6) 
			      	{nMonSinceFirstMolad +=  2;}
			        else
			          {
			        	if (nYearH >= 3) 
			        	{nMonSinceFirstMolad++  ;}
			          }
		            }
			     }
			   }
			}
		return nMonSinceFirstMolad;
	}
	
	/**
	 * Returns the Moonrise and Moonset times in an array of two strings.
	 * If there is no rise or set at the specified date, 
	 * the returned string is "--:--".
	 */
	public String[]  Moon(int day, int month, int year,double latitude , double longitude,double deltahour)
	{
	                    
		RiseSet[0] = "--:--" ;
		RiseSet[1] = "--:--" ;
		String st1,  st2 ;
							// test the input
		if ((day < 1 )|( day > 31)|(month<1)|(month>12)|(year <100))
		{RiseSet[0] = "Date error";
		 RiseSet[1] = "Date error" ;
		 return RiseSet;}
		else if ((year == 1582) & (month == 10) & ( day > 4) & (day < 15))
		{RiseSet[0] = "Date error";
		 RiseSet[1] = "Date error" ;
		 return RiseSet;}
		else
		{  
			B5 = latitude;
			L5 = longitude;
			L5 = L5/360;
			Z0 = -deltahour/24;
			D = day;
			Mo = month;
			Y = year;			

			x760();
			T = J - 2451545 + F ;
								//   LST AT 0H ZONE Time
			T0 = T / 36525;
			S = 24110.5 + 8640184.813 * T0;
			S += 86636.6 * Z0 + 86400 * L5 ;
			S = S / 86400 ;
			S -= Math.floor(S);
			T0 = S * 360 * DtoR;
			T = T + Z0 ;
								//     POSITION Loop
			
			for(int i=1 ; i< 4;i++)
			{
				x495() ;
				Ma[i][1]=A5;
				Ma[i][2]=D5;
				Ma[i][3]=R5;
				T=T+0.5;
			}
			
			if (Ma[2][1]<= Ma[1][1])
			{ Ma[2][1]=Ma[2][1]+P2 ;}
			
			if (Ma[3][1] <= Ma[2][1]) 
			{Ma[3][1]=Ma[3][1]+P2;}
			
			Z1=DtoR*(90.567-41.685/Ma[2][3]);
			S=Math.sin(B5*DtoR)  ;
			C=Math.cos(B5*DtoR);
			Z=Math.cos(Z1);
			
			A0=Ma[1][1];
			D0=Ma[1][2];	
							
			for(int Count=0 ; Count<24;Count++)
			{
				C0 = Count;
				P=(C0+1)/24;
				F0=Ma[1][1];
				F1=Ma[2][1];
				F2=Ma[3][1];
				
				A=F1-F0;
				B=F2-F1-A;
				A2=F0+P*(2*A+B*(2*P-1));
				F0=Ma[1][2];
				F1=Ma[2][2];
				F2=Ma[3][2];
				A=F1-F0;
				B=F2-F1-A;
				D2=F0+P*(2*A+B*(2*P-1));
						//'  TEST AN HOUR For AN EVENT
				L0=T0+C0*K1;
				L2=L0+K1;
				if (A2<A0) 
				{ A2=A2+P2 ;}
				H0=L0-A0;
				H2=L2-A2;
				H1=(H2+H0)/2   ;//       '  REM  HOUR ANGLE
				D1=(D2+D0)/2  ;//          ' REM  DEC
				
				if (C0==0) 
				{  V0=S*Math.sin(D0)+C*Math.cos(D0)*Math.cos(H0)-Z ;}
			
				V2= S *Math.sin(D2)+C*Math.cos(D2)*Math.cos(H2)-Z ;	
	
				if (V0/Math.abs(V0)!= V2/Math.abs(V2))
				{
					V1= S *Math.sin(D1)+C*Math.cos(D1)*Math.cos(H1)-Z;
					A=2*V2-4*V1+2*V0;
					B=4*V1-3*V0-V2;
					D=B*B-4*A*V0;
					if (D >= 0) 
					{
						D=Math.pow(D,0.5);
						if (V0<0 & V2>0)
						   {M8=1;}
						if (V0>0 & V2<0) 
						   {W8=1;}
						E=(-B+D)/(2*A);
						if( E>1 | E<0) 
							{E=(-B-D)/(2*A);}
						T3=C0+E+1/120 ; //             '  REM Round OFF
						H3=Math.floor(T3);
						M3=Math.floor((T3-H3)*60+0.5);
						if (M3 == 60)
						{
							H3 += 1 ;
							M3 = 0 ;
						}							
						if (V0<0 & V2>0)
						{
							if (H3 < 10 )
								st1 = "0" +  Integer.toString((int)H3) ;
							else 
								st1 = Integer.toString((int)H3) ;
						
							if (M3 < 10 )
								st2 = "0" +  Integer.toString((int)M3) ;
							else 
								st2 = Integer.toString((int)M3) ;
							RiseSet[0] = st1 + ":" + st2 ;
						}
						if (V0>0 & V2<0) 
						{
							if (H3 < 10 )
								st1 = "0" +  Integer.toString((int)H3) ;
							else 
								st1 = Integer.toString((int)H3) ;
						
							if (M3 < 10 )
								st2 = "0" +  Integer.toString((int)M3) ;
							else 
								st2 = Integer.toString((int)M3) ;
							RiseSet[1] = st1 + ":" + st2 ;
						}
					}
				}
				A0=A2;
				D0=D2;
				V0=V2;
			}

				//	x450  
			if (M8==0 & W8==0) 
				{
					if (V2<0) 
					{RiseSet[0] = "--:--" ;}
					if( V2>0) 
					{RiseSet[1] = "--:--";}
				}
			else 
				{
					if (M8==0)
					{RiseSet[0] = "--:--";}
					if (W8==0) 
					{RiseSet[1] = "--:--";}
				}
			M8 = 0;
			W8 = 0;
			return RiseSet ;
		}
	}
	
	private void x760()
	{
		double G = 1 ;
		if (Y < 1582) 
			{G = 0;}
		D1 = D;
		F = D - D1 - 0.5 ;
		J = -Math.floor(7*(Math.floor((Mo+9)/12)+Y)/4);
		if (G != 0) 
			{
				if (Mo != 9) 
				{
			     A = Math.abs(Mo-9);
			     S = (Mo-9)/A;
				 J3 = Math.floor(Y+S*Math.floor(A/7));
				}
			   else
			   { J3 = Y;}
			   
			   J3 = Math.floor(J3/100);
			   J3 = -Math.floor((J3 +1)* 3 / 4);
			}
		J = J + Math.floor(275 * Mo / 9) + D1 + G * J3;
		J = J + 1721027 + 2 * G + 367 * Y;
		if (F >= 0) 
			{
			F += 1;
			J -= 1;
			}
	}
	
	private void x495()
	{
			L=0.606434+0.03660110129*T;
			M=0.374897+0.03629164709*T;
			F=0.259091+0.03674819520*T;
			D=0.827362+0.03386319198*T;
			N=0.347343-0.00014709391*T;
			G=0.993126+0.00273777850*T;
			L -= Math.floor(L);
			M -= Math.floor(M);
			F -= Math.floor(F);
			D -= Math.floor(D);
			N -= Math.floor(N);
			G -= Math.floor(G);
			L *=  P2;
			M *=  P2;
			F *=  P2;
			D *=  P2;
			N *=  P2;
			G *=  P2;
			V=  0.39558*Math.sin(F+N);
			V=V+0.08200*Math.sin(F);
			V=V+0.03257*Math.sin(M-F-N);
			V=V+0.01092*Math.sin(M+F+N);
			V=V+0.00666*Math.sin(M-F);
			V=V-0.00644*Math.sin(M+F-2*D+N);
			V=V-0.00331*Math.sin(F-2*D+N);
			V=V-0.00304*Math.sin(F-2*D);
			V=V-0.00240*Math.sin(M-F-2*D-N);
			V=V+0.00226*Math.sin(M+F);
			V=V-0.00108*Math.sin(M+F-2*D);
			V=V-0.00079*Math.sin(F-N);
			V=V+0.00078*Math.sin(F+2*D+N);
			U=1-0.10828*Math.cos(M);
			U=U-0.01880*Math.cos(M-2*D);
			U=U-0.01479*Math.cos(2*D);
			U=U+0.00181*Math.cos(2*M-2*D);
			U=U-0.00147*Math.cos(2*M);
			U=U-0.00105*Math.cos(2*D-G);
			U=U-0.00075*Math.cos(M-2*D+G);
			W=0.10478*Math.sin(M);
			W=W-0.04105*Math.sin(2*F+2*N);
			W=W-0.02130*Math.sin(M-2*D);
			W=W-0.01779*Math.sin(2*F+N);
			W=W+0.01774*Math.sin(N);
			W=W+0.00987*Math.sin(2*D);
			W=W-0.00338*Math.sin(M-2*F-2*N);
			W=W-0.00309*Math.sin(G);
			W=W-0.00190*Math.sin(2*F);
			W=W-0.00144*Math.sin(M+N);
			W=W-0.00144*Math.sin(M-2*F-N);
			W=W-0.00113*Math.sin(M+2*F+2*N);
			W=W-0.00094*Math.sin(M-2*D+G);
			W=W-0.00092*Math.sin(2*M-2*D);
			
						//  COMPUTE RA, DEC, DIST
			S=W/Math.pow((U-V*V),0.5);
			A5=L+Math.atan(S/Math.pow((1-S*S),0.5));
			S=V/Math.pow(U,0.5);
			D5=Math.atan(S/Math.pow((1-S*S),0.5));
			R5 = 60.40974 * Math.pow(U,0.5);
	}
	
	/**
	 * Returns the Sunrise and Sunset times in an array of two strings.
	 * At Latitudes greater then 65 or smaller then -65 there are cases that 
	 * there is no sunrise and sunset. The result in this case is "--:--" 
	 */
	public String[]  Sun(int day,int month, int year,double latitude , double longitude,double deltahour)
	{
   
		// test the input
		if ((day < 1 )|( day > 31)|(month<1)|(month>12)|(year <100))
		{RiseSet[0] = "Date error";
		 RiseSet[1] = "Date error" ;}
		else if ((year == 1582) & (month == 10) & ( day > 4) & (day < 15))
		{RiseSet[0] = "Date error";
		 RiseSet[1] = "Date error" ;}
		else
		{
			int N1 = (int) (275 * month / 9) ;
			int N2 = (int)(Math.floor((month + 9) / 12)) ;
			int N3 = (int)(1 + Math.floor((year - 4 * Math.floor(year / 4) + 2) / 3));
			int N = (int)(N1 - N2 * N3 + day - 30) ;
			
			RiseSet[0] = tsun( 0, N, latitude, longitude, deltahour) ; // rise time
			RiseSet[1] = tsun( 1, N, latitude, longitude, deltahour) ; // set time
		}
		return RiseSet ;
	}
	
	
	private String tsun(int order,int N, double latit, double longit, double deltahour)
	{
		double RtoD = 180/Math.PI	;
        double DtoR = Math.PI/180 ;	
        double lngHour = longit / 15;
        double H ;
		double TT ;
		double UT;
		double localT ;
		int ht ;
		double mint ;
		
		String st1,st2 ;
        double t = N + (6 + 12 * order - lngHour) / 24;
		double M = (0.9856 * t) - 3.289;
		double L = M + (1.916 * Math.sin(DtoR*M)) + (0.020 * Math.sin(DtoR * 2 * M)) + 282.634;
		L = (L + 720 )% 360;
		double RA = RtoD*Math.atan(0.91764 * Math.tan(DtoR*L));
		RA = (RA + 720 )% 360;
		double Lquadrant  = (Math.floor( L/90)) * 90;
		double RAquadrant = (Math.floor(RA/90)) * 90;
		RA += Lquadrant - RAquadrant;
		RA = (RA + 720 )% 360;
		RA = RA / 15;
		double sinDec = 0.39782 * Math.sin(DtoR*L); //' this is in radians
		double cosDec = Math.cos(Math.asin(sinDec))   ; 
		double cosH = (Math.cos(DtoR*zenith) - (sinDec * Math.sin(DtoR*latit))) / (cosDec * Math.cos(DtoR*latit));
		if (cosH >  1) 
		{ RiseSet[order]= "--:--" ;}
		else
		{
			if (order == 0)
				H = 360 - RtoD * Math.acos(cosH);
			else
				H = RtoD * Math.acos(cosH);
			H = H / 15;
			TT = H + RA - 0.06571 * t - 6.622;
			UT = TT - lngHour;
			localT = (UT + deltahour + 48)% 24;
			ht = (int)(Math.floor (localT));
						
			mint = Math.floor((localT - Math.floor (localT)) *60 + 0.5 );
			if (mint == 60)
			{ht += 1 ;
			mint = 0 ;}
					
			if (ht < 10)
				st1 = "0" + Integer.toString(ht) ;
			else
				st1 = Integer.toString(ht) ;
			if (mint < 10.0 )
				st2 = "0" + Integer.toString((int)mint) ;
			else 
				st2 = Integer.toString((int)mint) ;
			RiseSet[order] = st1 + ":" + st2;
			if (st2.length() == 1) // case that mint is null
		  		RiseSet[order] = "--:--" ;
			
		}
		return RiseSet[order] ;
	}
	
	
	/**
	 * Returns the Julian Day.
	 * deltahour is the local time offset. 
	 */
	public double Julianday(int day, int month, int year, int hour,int minute, double deltahour ) throws ParseException 
	{
		long dt ;
		dt = DateTime.DateParse("01/01/1900") ;
		dt = DateTime.Add(dt, year - 1900, month- 1, day - 1) ;
		dt += (hour-deltahour)* DateTime.TicksPerHour + minute* DateTime.TicksPerMinute ;
		return  ((double)dt / DateTime.TicksPerDay) + 2440587.5;
	}
 
	/**
	 * Returns the Julian Day using DateTime parameter.
	 * datetime is the local time in DateTime ticks.
	 * deltahour is the local time offset. 
	 */
	public double Julianday2(long datetime, double deltahour ) 
	{
		long dt =  datetime - (long)(deltahour * DateTime.TicksPerHour) ;
		return  ((double)dt / DateTime.TicksPerDay) + 2440587.5; 
	}
	

	
	/**
	 * Returns the ECLIPTICAL LONGITUDE of the sun.
	 * @throws ParseException 
	 */
	public double GetSunLong(int day, int month, int year, int hour,int minute, double deltahour) throws ParseException
	{
		return sunlong(Julianday(day, month,year, hour, minute, deltahour) );
	}
	
	/**
	 * Returns the ECLIPTICAL LONGITUDE of the sun using Julian Day as parameter.
	 */
	public double GetSunLong2(double julianday)
	{
		return sunlong(julianday );
	}
	
	private double sunlong(double jd )
	{
		//double rad = 1/ DtoR ;
		double  ts ;	
		ts = (jd - 2451545 + 0.00074074) / 365.25 ; //'delta 64 seconds
		double l_sun = 280.4603 + 360.00769 * ts ;
		l_sun += (1.9146 - 0.00005 * ts) * Math.sin((359.991 * ts + 357.538) * DtoR);
		l_sun += 0.02 * Math.sin((719.981 * ts + 355.05) * DtoR);
		l_sun += 0.0048 * Math.sin((19.341 * ts + 234.95) * DtoR);
		l_sun += 0.002 * Math.sin((329.64 * ts + 247.1) * DtoR);
		l_sun += 0.0018 * Math.sin((4452.67 * ts + 297.8) * DtoR);
		l_sun += 0.0018 * Math.sin((0.2 * ts + 251.3) * DtoR);
		l_sun += 0.0015 * Math.sin((450.37 * ts + 343.2) * DtoR);
		l_sun += 0.0013 * Math.sin((225.18 * ts + 81.4) * DtoR);
		l_sun += 0.0008 * Math.sin((659.29 * ts + 132.5) * DtoR);
		l_sun += 0.0007 * Math.sin((90.38 * ts + 153.3) * DtoR);
		l_sun += 0.0007 * Math.sin((30.35 * ts + 206.8) * DtoR);
		l_sun += 0.0006 * Math.sin((337.18 * ts + 29.8) * DtoR);
		l_sun += 0.0005 * Math.sin((1.5 * ts + 207.4) * DtoR);
		l_sun += 0.0005 * Math.sin((22.81 * ts + 291.2) * DtoR);
		l_sun += 0.0004 * Math.sin((315.56 * ts + 234.9) * DtoR);
		l_sun += 0.0004 * Math.sin((299.3 * ts + 157.3) * DtoR);
		l_sun += 0.0004 * Math.sin((720.02 * ts + 21.1) * DtoR);
		l_sun += 0.0003 * Math.sin((1079.97 * ts + 352.5) * DtoR);
		l_sun += 0.0003 * Math.sin((44.43 * ts + 329.7) * DtoR);
	//	l_sun = Ang(l_sun) / rad	;	//	'ECLIPTICAL LONGITUDE OF THE SUN
		l_sun = (l_sun -  Math.floor(l_sun / 360) * 360) * DtoR ;
		if (l_sun < 0)
		  l_sun += P2;
		return l_sun ;
	}
	
	/**
	 * Returns an array of 3 doubles:
	 * - Sun's distance in Astronomical Units AU 
	 * (1 Astronomical Unit = 149,598,000 kilometers)
	 * - The Sun's angular diameter
	 * - The sun's brightness (magnitude)
	 * @throws ParseException 
	 */
	public double[] GetSunData(int day, int month, int year, int hour,int minute, double deltahour) throws ParseException
	{
		return sundata(Julianday(day, month,  year, hour, minute, deltahour) );
	}
	
	/**
	 * Returns an array of 3 doubles:
	 * - Sun's distance in Astronomical Units AU.
	 * (1 Astronomical Unit = 149,598,000 kilometers)
	 * - The Sun's angular diameter
	 * - The sun's brightness (magnitude)
	 *  using Julian Day as parameter.
	 */
	public double[] GetSunData2(double julianday)
	{
		return sundata(julianday );
	}
	
	private double [] sundata(double jd )
	{
		double  ts ;
		double [] data = new double[3];
		ts = (jd - 2451545 + 0.00074074) / 365.25 ; //'delta 64 seconds
		double r_sun = (0.007256 - 0.0000002 * ts) * Math.sin((359.991 * ts + 267.54) * DtoR);
		r_sun += 0.000091 * Math.sin((719.98 * ts + 265.1) * DtoR);
		r_sun += 0.00003 * Math.sin((90) * DtoR);
		r_sun += 0.000013 * Math.sin((4452.67 * ts + 27.8) * DtoR);
		r_sun += 0.000007 * Math.sin((450.4 * ts + 254) * DtoR);
		r_sun += 0.000007 * Math.sin((329.6 * ts + 156) * DtoR);
		r_sun = Math.pow(10,r_sun)	;				//	'SUN DISTANCE IN AU
		data[0] = r_sun ;
		data[1] = 31.9876667 / r_sun ;  //'in arc min  'SUN ANGULAR DIAMETER
		data[2] = -26.74 + 5 * Math.log10(r_sun * r_sun);	//'SUN BRIGHTNESS (MAGNITUDE)
		return data;
	}	

	/**
	 * Returns the Moon's Ecliptical Longitude.
	 * @throws ParseException 
	 */
	public double GetMoonLong(int day, int month, int year, int hour,int minute, double deltahour) throws ParseException
	{
		return moonlong(Julianday(day, month,  year,hour, minute, deltahour) );
	}
	
	/**
	 * Returns the Moon's Ecliptical Longitude, using Julian day as parameter.
	 */	
	public double GetMoonLong2(double julianday)
	{
		return moonlong(julianday );
	}

	private double moonlong(double jd )
	{
		double ts ;
		ts = (jd - 2451545 + 0.00074074) / 36525 ;// 'delta 64 seconds
		double l_moon = 218.3166 + 481267.8811 * ts - 0.0015 * ts * ts ;
		l_moon += 6.2888 * Math.cos((477198.868 * ts + 44.963) * DtoR) ;
		l_moon += 1.274 * Math.cos((413335.35 * ts + 10.74) * DtoR);
		l_moon += 0.6583 * Math.cos((890534.22 * ts + 145.7) * DtoR);
		l_moon += 0.2136 * Math.cos((954397.74 * ts + 179.93) * DtoR);
		l_moon += 0.1851 * Math.cos((35999.05 * ts + 87.53) * DtoR);
		l_moon += 0.1144 * Math.cos((966404 * ts + 276.5) * DtoR);
		l_moon += 0.0588 * Math.cos((63863.5 * ts + 124.2) * DtoR);
		l_moon += 0.0571 * Math.cos((377336.3 * ts + 13.2) * DtoR);
		l_moon += 0.0533 * Math.cos((1367733.1 * ts + 280.7) * DtoR);
		l_moon += 0.0458 * Math.cos((854535.2 * ts + 148.2) * DtoR);
		l_moon += 0.0409 * Math.cos((441199.8 * ts + 47.4) * DtoR);
		l_moon += 0.0347 * Math.cos((445267.1 * ts + 27.9) * DtoR);
		l_moon += 0.0304 * Math.cos((513197.9 * ts + 222.5) * DtoR);
		l_moon += 0.0154 * Math.cos((75870 * ts + 41) * DtoR);
		l_moon += 0.0125 * Math.cos((1443603 * ts + 52) * DtoR);
		l_moon += 0.011 * Math.cos((489205 * ts + 142) * DtoR);
		l_moon += 0.0107 * Math.cos((1303870 * ts + 246) * DtoR);
		l_moon += 0.01 * Math.cos((1431597 * ts + 315) * DtoR);
		l_moon += 0.0085 * Math.cos((826671 * ts + 111) * DtoR);
		l_moon += 0.0079 * Math.cos((449334 * ts + 188) * DtoR);
		l_moon += 0.0068 * Math.cos((926533 * ts + 323) * DtoR);
		l_moon += 0.0052 * Math.cos((31932 * ts + 107) * DtoR);
		l_moon += 0.005 * Math.cos((481266 * ts + 205) * DtoR);
		l_moon += 0.004 * Math.cos((1331734 * ts + 283) * DtoR);
		l_moon += 0.004 * Math.cos((1844932 * ts + 56) * DtoR);
		l_moon += 0.004 * Math.cos((133 * ts + 29) * DtoR);
		l_moon += 0.0038 * Math.cos((1781068 * ts + 21) * DtoR);
		l_moon += 0.0037 * Math.cos((541062 * ts + 259) * DtoR);
		l_moon += 0.0028 * Math.cos((1934 * ts + 145) * DtoR);
		l_moon += 0.0027 * Math.cos((918399 * ts + 182) * DtoR);
		l_moon += 0.0026 * Math.cos((1379739 * ts + 17) * DtoR);
		l_moon += 0.0024 * Math.cos((99863 * ts + 122) * DtoR);
		l_moon += 0.0023 * Math.cos((922466 * ts + 163) * DtoR);
		l_moon += 0.0022 * Math.cos((818536 * ts + 151) * DtoR);
		l_moon += 0.0021 * Math.cos((990397 * ts + 357) * DtoR);
		l_moon += 0.0021 * Math.cos((71998 * ts + 85) * DtoR);
		l_moon += 0.0021 * Math.cos((341337 * ts + 16) * DtoR);
		l_moon += 0.0018 * Math.cos((401329 * ts + 274) * DtoR);
		l_moon += 0.0016 * Math.cos((1856938 * ts + 152) * DtoR);
		l_moon += 0.0012 * Math.cos((1267871 * ts + 249) * DtoR);
		l_moon += 0.0011 * Math.cos((1920802 * ts + 186) * DtoR);
		l_moon += 0.0009 * Math.cos((858602 * ts + 129) * DtoR);
		l_moon += 0.0008 * Math.cos((1403732 * ts + 98) * DtoR);
		l_moon += 0.0007 * Math.cos((790672 * ts + 114) * DtoR);
		l_moon += 0.0007 * Math.cos((405201 * ts + 50) * DtoR);
		l_moon += 0.0007 * Math.cos((485333 * ts + 186) * DtoR);
		l_moon += 0.0007 * Math.cos((27864 * ts + 127) * DtoR);
		l_moon += 0.0006 * Math.cos((111869 * ts + 38) * DtoR);
		l_moon += 0.0006 * Math.cos((2258267 * ts + 156) * DtoR);
		l_moon += 0.0005 * Math.cos((1908795 * ts + 90) * DtoR);
		l_moon += 0.0005 * Math.cos((1745069 * ts + 24) * DtoR);
		l_moon += 0.0005 * Math.cos((509131 * ts + 242) * DtoR);
		l_moon += 0.0004 * Math.cos((39871 * ts + 223) * DtoR);
		l_moon += 0.0004 * Math.cos((12006 * ts + 187) * DtoR);
		l_moon = l_moon * DtoR;
		l_moon = l_moon - Math.floor(l_moon / P2) * P2;	//''ECLIPTICAL LONGITUDE OF THE MOON
		if (l_moon < 0) 
			l_moon += P2;
		return l_moon ;
	}
	
	/**
	 * Returns the Moon's Latitude.
	 * @throws ParseException 
	 */	
	public double GetMoonLat(int day, int month, int year, int hour,int minute, double deltahour) throws ParseException
	{
		return moonlat(Julianday(day, month,  year, hour, minute, deltahour) );
	}
	/**
	 * Returns the Moon's Latitude, using Julian day as parameter.
	 */	
	public double GetMoonLat2(double julianday)
	{
		return moonlat(julianday );
	}
		
	private double moonlat(double jd )
	{
		double ts ;
		ts = (jd - 2451545 + 0.00074074) / 36525 ;// 'delta 64 seconds
		double b_moon = 5.1281 * Math.cos((483202.019 * ts + 3.273) * DtoR);
		b_moon += 0.2806 * Math.cos((960400.89 * ts + 138.24) * DtoR);
		b_moon += 0.2777 * Math.cos((6003.15 * ts + 48.31) * DtoR);
		b_moon += 0.1733 * Math.cos((407332.2 * ts + 52.43) * DtoR);
		b_moon += 0.0554 * Math.cos((896537.4 * ts + 104) * DtoR);
		b_moon += 0.0463 * Math.cos((69866.7 * ts + 82.5) * DtoR);
		b_moon += 0.0326 * Math.cos((1373736.2 * ts + 239) * DtoR);
		b_moon += 0.0172 * Math.cos((1437599.8 * ts + 273.2) * DtoR);
		b_moon += 0.0093 * Math.cos((884531 * ts + 187) * DtoR);
		b_moon += 0.0088 * Math.cos((471196 * ts + 87) * DtoR);
		b_moon += 0.0082 * Math.cos((371333 * ts + 55) * DtoR);
		b_moon += 0.0043 * Math.cos((547066 * ts + 217) * DtoR);
		b_moon += 0.0042 * Math.cos((1850935 * ts + 14) * DtoR);
		b_moon += 0.0034 * Math.cos((443331 * ts + 230) * DtoR);
		b_moon += 0.0025 * Math.cos((860538 * ts + 106) * DtoR);
		b_moon += 0.0022 * Math.cos((481268 * ts + 308) * DtoR);
		b_moon += 0.0022 * Math.cos((1337737 * ts + 241) * DtoR);
		b_moon += 0.0021 * Math.cos((105866 * ts + 80) * DtoR);
		b_moon += 0.0019 * Math.cos((924402 * ts + 141) * DtoR);
		b_moon += 0.0018 * Math.cos((820668 * ts + 153) * DtoR);
		b_moon += 0.0018 * Math.cos((519201 * ts + 181) * DtoR);
		b_moon += 0.0018 * Math.cos((1449606 * ts + 10) * DtoR);
		b_moon += 0.0015 * Math.cos((42002 * ts + 46) * DtoR);
		b_moon += 0.0015 * Math.cos((928469 * ts + 121) * DtoR);
		b_moon += 0.0015 * Math.cos((996400 * ts + 316) * DtoR);
		b_moon += 0.0014 * Math.cos((29996 * ts + 129) * DtoR);
		b_moon += 0.0013 * Math.cos((447203 * ts + 6) * DtoR);
		b_moon += 0.0013 * Math.cos((37935 * ts + 65) * DtoR);
		b_moon += 0.0011 * Math.cos((1914799 * ts + 48) * DtoR);
		b_moon += 0.001 * Math.cos((1297866 * ts + 288) * DtoR);
		b_moon += 0.0009 * Math.cos((1787072 * ts + 340) * DtoR);
		b_moon += 0.0008 * Math.cos((972407 * ts + 235) * DtoR);
		b_moon += 0.0007 * Math.cos((1309873 * ts + 205) * DtoR);
		b_moon += 0.0006 * Math.cos((559072 * ts + 134) * DtoR);
		b_moon += 0.0006 * Math.cos((1361730 * ts + 322) * DtoR);
		b_moon += 0.0005 * Math.cos((848532 * ts + 190) * DtoR);
		b_moon += 0.0005 * Math.cos((419339 * ts + 149) * DtoR);
		b_moon += 0.0005 * Math.cos((948395 * ts + 222) * DtoR);
		b_moon += 0.0004 * Math.cos((2328134 * ts + 149) * DtoR);
		b_moon += 0.0004 * Math.cos((1024264 * ts + 352) * DtoR);
		b_moon = b_moon * DtoR	;		//	'ECLIPTICAL LATITUDE OF THE MOON
		return b_moon ;
	}
	/**
	 * Returns the Moon's distance in Km.
	 * @throws ParseException 
	 */
	public double GetMoonDistance(int day, int month, int year, int hour,int minute, double deltahour) throws ParseException
	{
		double temp =  moondistance(Julianday(day, month,  year, hour, minute, deltahour) );
		return 6378.14 / Math.sin(temp)	; //'MOON DISTANCE in km
	}
	
	/**
	 * Returns the Moon's distance in Km, using Julian day as parameter.
	 */
	public double GetMoonDistance2(double julianday)
	{
		double temp =  moondistance(julianday );
		return 6378.14 / Math.sin(temp)	; //'MOON DISTANCE in km
	}
		
	private double moondistance(double jd )
	{
		double ts ;
		ts = (jd - 2451545 + 0.00074074) / 36525 ;// 'delta 64 seconds
		double p_moon = 0.950725;
		p_moon += 0.05182 * Math.cos((477198.868 * ts + 134.963) * DtoR);
		p_moon += 0.00953 * Math.cos((413335.35 * ts + 100.74) * DtoR);
		p_moon += 0.007842 * Math.cos((890534.22 * ts + 235.7) * DtoR);
		p_moon += 0.002824 * Math.cos((954397.74 * ts + 269.93) * DtoR);
		p_moon += 0.000858 * Math.cos((1367733.1 * ts + 10.7) * DtoR);
		p_moon += 0.000531 * Math.cos((854535.2 * ts + 238.2) * DtoR);
		p_moon += 0.0004 * Math.cos((377336.3 * ts + 103.2) * DtoR);
		p_moon += 0.000319 * Math.cos((441199.8 * ts + 137.4) * DtoR);
		p_moon += 0.000271 * Math.cos((445267 * ts + 118) * DtoR);
		p_moon += 0.000263 * Math.cos((513198 * ts + 312) * DtoR);
		p_moon += 0.000197 * Math.cos((489205 * ts + 232) * DtoR);
		p_moon += 0.000173 * Math.cos((1431597 * ts + 45) * DtoR);
		p_moon += 0.000167 * Math.cos((1303870 * ts + 336) * DtoR);
		p_moon += 0.000111 * Math.cos((35999 * ts + 178) * DtoR);
		p_moon += 0.000103 * Math.cos((826671 * ts + 201) * DtoR);
		p_moon += 0.000084 * Math.cos((63864 * ts + 214) * DtoR);
		p_moon += 0.000083 * Math.cos((926533 * ts + 53) * DtoR);
		p_moon += 0.000078 * Math.cos((1844932 * ts + 146) * DtoR);
		p_moon += 0.000078 * Math.cos((1781068 * ts + 111) * DtoR);
		p_moon += 0.000064 * Math.cos((1331734 * ts + 13) * DtoR);
		p_moon += 0.000063 * Math.cos((449334 * ts + 278) * DtoR);
		p_moon += 0.000041 * Math.cos((481266 * ts + 295) * DtoR);
		p_moon += 0.000034 * Math.cos((918399 * ts + 272) * DtoR);
		p_moon += 0.000033 * Math.cos((541062 * ts + 349) * DtoR);
		p_moon += 0.000031 * Math.cos((922466 * ts + 253) * DtoR);
		p_moon += 0.00003 * Math.cos((75870 * ts + 131) * DtoR);
		p_moon += 0.000029 * Math.cos((990397 * ts + 87) * DtoR);
		p_moon += 0.000026 * Math.cos((818536 * ts + 241) * DtoR);
		p_moon += 0.000023 * Math.cos((553069 * ts + 266) * DtoR);
		p_moon += 0.000019 * Math.cos((1267871 * ts + 339) * DtoR);
		p_moon += 0.000013 * Math.cos((1403732 * ts + 188) * DtoR);
		p_moon += 0.000013 * Math.cos((341337 * ts + 106) * DtoR);
		p_moon += 0.000013 * Math.cos((401329 * ts + 4) * DtoR);
		p_moon += 0.000012 * Math.cos((2258267 * ts + 246) * DtoR);
		p_moon += 0.000011 * Math.cos((1908795 * ts + 180) * DtoR);
		p_moon += 0.000011 * Math.cos((858602 * ts + 219) * DtoR);
		p_moon += 0.00001 * Math.cos((1745069 * ts + 114) * DtoR);
		p_moon = p_moon * DtoR;
		return p_moon;
	}
		
	/**
	 * Returns an array of 5 doubles:
	 * - Moon's phase in radians.
	 * - Moon's Age in days
	 * - Moon's angular diameter in arcminutes
	 * - Moon's elongation in radians
	 * - Moon's brightness
	 */
	public double[] GetMoonData(int day, int month, int year, int hour,int minute, double deltahour) throws ParseException
	{
		return moondata(Julianday(day,  month, year,  hour, minute, deltahour)) ;
	}
		
		/**
		 * Returns an array of 5 doubles:
		 * - Moon's phase in radians.
		 * - Moon's Age in days
		 * - Moon's angular diameter in arcminutes
		 * - Moon's elongation in radians
		 * - Moon's brightness
		 *  using Julian Day as parameter.
		 */
	public double[] GetMoonData2(double julianday)
	{
		return moondata(julianday) ;
	}
		
	private double [] moondata(double jd)
	{
		double [] data = new double[5];
		double lms = moonlong(jd ) - sunlong(jd ) ;
		double mage = lms - Math.floor(lms / P2) * P2 ;
		double ph = Math.acos(Math.cos(mage)); // MOON PHASE IN RADIANS
		data[0] = ph ;
		mage = mage / 0.21276871038558 ; // HERE MOON AGE IN DAYS
		data[1] = mage ;
		double sd_moon = 0.272493 * moondistance(jd ) * 120 /DtoR ; //in arcmin,  MOON ANGULAR DIAMETER
		data[2] = sd_moon ;
		double el_moon = Math.acos(Math.cos(moonlat(jd )) * Math.cos(lms)); //	MOON ELONGATION IN RADIANS
		data[3] = el_moon ;
		double phg, ph4 ;
		phg = ph /DtoR;
		ph4 = 180 - phg;
		ph4 = ph4 * ph4 * ph4 * ph4 ;
		double mv_moon = -12.73 + 0.026 * Math.abs(180 - phg) + 0.000000004 * ph4 ; //MOON BRIGHTNESS
		data[4] = mv_moon ;
		return data ;
	}
	
	
	
	// This file is available in electronic form at http://www.psa.es/sdg/sunpos.htm


	/**
	 * Timenow is the local time at the lat-long point
	 * offset is the local offset in hours 
	 * Returns an array of 2 doubles, in degrees:
	 * - sun's azimuth
	 * - sun's elevation
	 */
	public double[] SunPosition( double Lat, double Lon, long Timenow, double offset ) 
	{
		// Main variables

		double twopi = 2*Math.PI ;
		double rad  = Math.PI/180 ;
		double dEarthMeanRadius  =   6371.01	;// In km
		double dAstronomicalUnit  =  149597890.0 ;	// In km
		double dElapsedJulianDays;
		double dDecimalHours;
		double dEclipticLongitude;
		double dEclipticObliquity;
		double dRightAscension;
		double dDeclination;

		// Auxiliary variables
		double dY;
		double dX;

		// Calculate difference in days between the current Julian Day 
		// and JD 2451545.0, which is noon 1 January 2000 Universal Time
		{
			double dJulianDate;
			long  liAux1;
			long  liAux2;
			// Calculate time of the day in UT decimal hours
		//	long offsetTicks = (long) (DateTime.getTimeZoneOffset() * DateTime.TicksPerHour);
			long offsetTicks = (long) (offset * DateTime.TicksPerHour);
			long UTCTimenow = Timenow - offsetTicks ;
			dDecimalHours = DateTime.GetHour(UTCTimenow ) + DateTime.GetMinute(UTCTimenow )/60 + DateTime.GetSecond(UTCTimenow )/3600.0 ;
			// Calculate current Julian Day
			liAux1 = (DateTime.GetMonth(Timenow) - 14) / 12  ;
			liAux2 = (1461 * (DateTime.GetYear(Timenow) + 4800 + liAux1)) / 4 + 
					  (367*(DateTime.GetMonth(Timenow) 
					  - 2 -12 * liAux1)) / 12- (3 * ((DateTime.GetYear(Timenow) + 4900 
				  + liAux1) / 100)) / 4 + DateTime.GetDayOfMonth(Timenow) - 32075 ;
			dJulianDate = (double)(liAux2) - 0.5 + dDecimalHours / 24.0;
			// Calculate difference between current Julian Day and JD 2451545.0 
			dElapsedJulianDays = dJulianDate - 2451545.0;
		}

		// Calculate ecliptic coordinates (ecliptic longitude and obliquity of the 
		// ecliptic in radians but without limiting the angle to be less than 2*Pi 
		// (i.e., the result may be greater than 2*Pi)
		{
			double dMeanLongitude;
			double dMeanAnomaly;
			double dOmega;
			dOmega = 2.1429 - 0.0010394594 * dElapsedJulianDays;
			dMeanLongitude = 4.8950630 + 0.017202791698 * dElapsedJulianDays; // Radians
			dMeanAnomaly = 6.2400600 + 0.0172019699 * dElapsedJulianDays;
			dEclipticLongitude = dMeanLongitude + 0.03341607 * Math.sin(( dMeanAnomaly ) 
					+ 0.00034894 * Math.sin( 2*dMeanAnomaly ) - 0.0001134 - 0.0000203 * Math.sin(dOmega));
			dEclipticObliquity = 0.4090928 - 6.2140e-9 * dElapsedJulianDays	+ 0.0000396 * Math.cos(dOmega);
		}

		// Calculate celestial coordinates ( right ascension and declination ) in radians 
		// but without limiting the angle to be less than 2*Pi (i.e., the result may be 
		// greater than 2*Pi)
		{
			double dSin_EclipticLongitude;
			dSin_EclipticLongitude= Math.sin( dEclipticLongitude );
			dY = Math.cos( dEclipticObliquity ) * dSin_EclipticLongitude;
			dX = Math.cos( dEclipticLongitude );
			dRightAscension = Math.atan2( dY,dX );
			if( dRightAscension < 0.0 ) dRightAscension = dRightAscension + twopi ;
			dDeclination = Math.asin( Math.sin( dEclipticObliquity ) * dSin_EclipticLongitude );
		}

		// Calculate local coordinates ( azimuth and zenith angle ) in degrees
		{
			double dGreenwichMeanSiderealTime;
			double dLocalMeanSiderealTime;
			double dLatitudeInRadians;
			double dHourAngle;
			double dCos_Latitude;
			double dSin_Latitude;
			double dCos_HourAngle;
			double dParallax;
			double dZenithAngle ;
			double dAzimuth ;
			dGreenwichMeanSiderealTime = 6.6974243242 + 0.0657098283 * dElapsedJulianDays + dDecimalHours;
			dLocalMeanSiderealTime = (dGreenwichMeanSiderealTime * 15 + Lon) * rad;
			dHourAngle = dLocalMeanSiderealTime - dRightAscension;
			dLatitudeInRadians = Lat * rad;
			dCos_Latitude = Math.cos( dLatitudeInRadians );
			dSin_Latitude = Math.sin( dLatitudeInRadians );
			dCos_HourAngle= Math.cos( dHourAngle );
			dZenithAngle = (Math.acos( dCos_Latitude * dCos_HourAngle * Math.cos(dDeclination) + Math.sin( dDeclination ) * dSin_Latitude));
			dY = -Math.sin( dHourAngle );
			dX = Math.tan( dDeclination ) * dCos_Latitude - dSin_Latitude * dCos_HourAngle;
			dAzimuth = Math.atan2( dY, dX );
			if ( dAzimuth < 0.0 ) 
				dAzimuth = dAzimuth + twopi;
			dAzimuth = dAzimuth / rad;
			// Parallax Correction
			dParallax = (dEarthMeanRadius / dAstronomicalUnit) * Math.sin(dZenithAngle);
			dZenithAngle = (dZenithAngle + dParallax) / rad;
			double[] dbl = new double[2];
			dbl[0] = dAzimuth ;
			dbl[1] = 90 - dZenithAngle ;
			return dbl ;
		}
	}
}
	
